SystemOrganization addCategory: #'Magma-Tools-Buffer Editor'!
SystemOrganization addCategory: #'Magma-Tools-Exceptions'!
SystemOrganization addCategory: #'Magma-Tools-Health'!
SystemOrganization addCategory: #'Magma-Tools-Info'!
SystemOrganization addCategory: #'Magma-Tools-Performance'!
SystemOrganization addCategory: #'Magma-Tools-Presentation'!
SystemOrganization addCategory: #'Magma-Tools-Statistics'!
SystemOrganization addCategory: #'Magma-Tools-Stress Tester'!
SystemOrganization addCategory: #'Magma-Tools-Upgrade'!

!MaVariableObjectBuffer methodsFor: '*magma-tools' stamp: 'cmm 3/19/2007 12:26'!
hasValidReferencesFor: aMaObjectFiler 
	self instVarsDoWithIndex: 
		[ : eachOid : x | 
		(self 
			isValidOid: eachOid
			for: aMaObjectFiler) ifFalse: [ ^ false ] ].
	^ true! !

MagmaUserError subclass: #MagmaToolsUserError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma-Tools-Exceptions'!

MaObject subclass: #MagmaBenchmarker
	instanceVariableNames: 'fixture session'
	classVariableNames: 'HardwareInfo Port'
	poolDictionaries: ''
	category: 'Magma-Tools-Performance'!

!MagmaBenchmarker class methodsFor: 'private' stamp: 'cmm 2/10/2012 15:40'!
benchName
	^ FileDirectory default fullNameFor: '_magma-bench'! !

!MagmaBenchmarker class methodsFor: 'hardware info' stamp: 'cmm 12/7/2011 21:50'!
captureHardwareDetails
	| answer |
	[ answer := UIManager default
		request: 'Please enter a brief description of your hardware separated by commas ' , self hardwareAttributes
		initialAnswer: (HardwareInfo ifNil: [ '' ]).
	(answer subStrings: ',') size < (self hardwareAttributes subStrings: ',') size ] whileTrue: [ self inform: 'All of those things are important, please enter them all.' ].
	^ answer! !

!MagmaBenchmarker class methodsFor: 'instance creation' stamp: 'cmm 7/5/2008 14:10'!
connectSessionTo: hostString useWriteBarrier: useWriteBarrier 
	| session |
	session := MagmaSession 
		host: hostString
		port: self port.
	session allowWriteBarrier: useWriteBarrier.
	session connectAs: 'benchTest'.
	^ self session: session! !

!MagmaBenchmarker class methodsFor: 'instance creation' stamp: 'cmm 7/5/2008 14:10'!
forBenchmarkHost: hostString withWriteBarrier: useWriteBarrier 
	"This is a 'one-click' run of the benchmarks that exercises as much of the code as possible, while excluding network latency by using a hostString of 'localhost'."
	^ self 
		connectSessionTo: hostString
		useWriteBarrier: useWriteBarrier! !

!MagmaBenchmarker class methodsFor: 'instance creation' stamp: 'cmm 6/10/2005 16:58'!
forLocalBaseline: useWriteBarrier
	"This is a 'one-click' run of the benchmarks that exercises most of the Magma code without the serialization/materialization cost of Ma client server."

	| session |
	self resetRepository.
	session := MagmaSession openLocal: self benchName.
	session allowWriteBarrier: useWriteBarrier.
	session connectAs: 'benchTest'.
	^ self session: session! !

!MagmaBenchmarker class methodsFor: 'hardware info' stamp: 'cmm 3/23/2005 17:16'!
hardwareAttributes

	^ 'computer, cpu, speed, memory, disk'! !

!MagmaBenchmarker class methodsFor: 'hardware info' stamp: 'cmm 3/23/2005 17:08'!
hardwareInfo

	^ HardwareInfo := self captureHardwareDetails! !

!MagmaBenchmarker class methodsFor: 'network baseline' stamp: 'cmm 7/6/2008 20:18'!
port
	"The port the benchmark host will listen on."
	^ Port ifNil: [ Port := 51969 ]! !

!MagmaBenchmarker class methodsFor: 'network baseline' stamp: 'cmm 7/6/2008 20:18'!
port: anInteger
	"Set the port the benchmark host will listen on."
	Port := anInteger! !

!MagmaBenchmarker class methodsFor: 'private' stamp: 'cmm 7/16/2007 22:08'!
resetRepository
	[ MagmaRepositoryController delete: self benchName ] 
		on: MagmaDeleteWarning
		do: [ : warning | warning resume ].
	MagmaRepositoryController 
		create: self benchName
		root: Dictionary new! !

!MagmaBenchmarker class methodsFor: 'quick baseline run' stamp: 'cmm 11/13/2006 23:31'!
runLocalBaseline: useWriteBarrier 
	"This is a 'one-click' run of the benchmarks that exercises most of the Magma code without the serialization/materialization cost of Ma client server."
	| benchmarker |
	^ [ (benchmarker := self forLocalBaseline: useWriteBarrier) runAll ] ensure: [ benchmarker session disconnectAndClose ]! !

!MagmaBenchmarker class methodsFor: 'network baseline' stamp: 'cmm 11/27/2011 15:30'!
runRemoteBaselineAgainst: hostString withWriteBarrier: useWriteBarrier 
	"This is a 'one-click' run of the benchmarks over a network, the results includes the network latency."
	| benchmarker |
	^ 
	[ (benchmarker := self 
		forBenchmarkHost: hostString
		withWriteBarrier: useWriteBarrier) runAll ] ensure: [ benchmarker session disconnectAndClose ]! !

!MagmaBenchmarker class methodsFor: 'network baseline' stamp: 'cmm 11/27/2011 15:49'!
runRemoteBaselineWithWriteBarrier: useWriteBarrier 
	"This is a 'one-click' run of the benchmarks over a network, the results includes the network latency.
	Before running this method:
		1) Save this image.
		2) Save this image as another name (e.g., _bench-server.image).
		3) Relaunch this image.
		4) Run this method in the relaunched image (print it!!)."
	^ self
		runRemoteBaselineAgainst: 'localhost'
		withWriteBarrier: useWriteBarrier! !

!MagmaBenchmarker class methodsFor: 'private' stamp: 'cmm 3/23/2005 23:27'!
runWith: aMagmaSession

	^ (self session: aMagmaSession) runAll! !

!MagmaBenchmarker class methodsFor: 'instance creation' stamp: 'cmm 3/15/2005 22:21'!
session: aMagmaSession

	^ self new
		session: aMagmaSession ;
		yourself! !

!MagmaBenchmarker class methodsFor: 'network baseline' stamp: 'cmm 6/26/2008 10:51'!
startBenchmarkHost
	| console |
	self resetRepository.
	console := MagmaServerConsole new open: self benchName.
	console processOn: self port.
	console inspect! !

!MagmaBenchmarker methodsFor: 'benchmarks-write' stamp: 'cmm 11/27/2011 11:53'!
a1001BufferCommit
	"How fast can it commit when there are 1001 objects to write in each commit?"
	| count |
	count := 0.
	session abort; begin.
	self focusOn: #oneThousandElementArrayOfObjectsFixture.
	^ [ self oneThousandElementArrayOfObjectsFixture do: [ : eachArray | eachArray at: 1 put: (count := count + 1) ].
	session commitAndBegin ] bench! !

!MagmaBenchmarker methodsFor: 'benchmarks-write' stamp: 'cmm 11/27/2011 15:27'!
aOneMillionObjectPointerCommit
	"How fast can it write 1000 1000-element Arrays?"
	| toggle |
	toggle := false.
	session
		 abort ;
		 begin.
	self focusOn: #oneMillionObjectPointersFixture.
	^ [ self oneMillionObjectPointersFixture do:
		[ : eachArray | eachArray
			at: 1
			put: (toggle := toggle not) ].
	session commitAndBegin ] bench! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmacollections' stamp: 'cmm 12/8/2009 10:36'!
addThousandsOfObjectsAtOnce
	"This tests how long it takes to add a fixed number of objects, one at a time, to a MagmaCollection with no indexes."
	self focusOn: #magmaCollectionFixture.
	session abort.	"ensure not in a transaction."
	^ ((Time millisecondsToRun: 
		[ session commit: 
			[ 1 
				to: self thousands
				do: [ : n | self magmaCollectionFixture add: Array new ] ] ]) / 1000.0) printString , ' seconds.'! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmacollections' stamp: 'cmm 12/8/2009 10:37'!
addThousandsOfObjectsCommitEvery: batchSize 
	"This tests how long it takes to add a fixed number of objects, one at a time, to a MagmaCollection with no indexes."
	"Ensure the collection is already created, so we only measure the add pace."
	session commit: [ self magmaCollectionFixture ].
	self focusOn: #magmaCollectionFixture.
	Smalltalk garbageCollect.
	session
		finalizeOids ;
		abort ;
		begin.
	^ ((Time millisecondsToRun: 
		[ 1 
			to: self thousands
			do: 
				[ : n | 
				self magmaCollectionFixture add: Array new.
				n \\ batchSize = 0 ifTrue: [ session commitAndBegin ] ].
		[ session inTransaction ] whileTrue: [ session commit ] ]) / 1000.0) printString , ' seconds.'! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmacollections' stamp: 'cmm 8/17/2005 22:24'!
addThousandsOfObjectsOneHundredAtATime
	"This tests how long it takes to add a fixed number of objects to a MagmaCollection with no indexes."

	^ self addThousandsOfObjectsCommitEvery: 100! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmacollections' stamp: 'cmm 8/17/2005 22:19'!
addThousandsOfObjectsTenAtATime
	"This tests how long it takes to add a fixed number of objects to a MagmaCollection with no indexes."

	^ self addThousandsOfObjectsCommitEvery: 10! !

!MagmaBenchmarker methodsFor: 'accessing' stamp: 'cmm 1/11/2007 21:45'!
allCategories
	"These selectors return the array of selectors to run for that category."
	^#( connectionTests readTests writeTests magmaArrayTests magmaCollectionTests )! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/15/2005 22:24'!
benchReadOf: oid

	^ [ session realObjectFor: oid ] bench! !

!MagmaBenchmarker methodsFor: 'benchmarks-write' stamp: 'cmm 1/11/2007 21:52'!
connectionTests
	^ #(timeToDisconnect timeToConnect)! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 8/17/2005 22:13'!
focusOn: fixtureSelector

	(fixture keys copyWithout: fixtureSelector) do:
		[ : each |
		session stubOut: (fixture at: each) ].
	Smalltalk garbageCollect.
	session finalizeOids.
	(self perform: fixtureSelector) yourself "ensure proxy-materialized."! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmaarrays' stamp: 'cmm 3/24/2005 00:23'!
getMagmaArraySize

	| array |
	array := fixture at: #magmaArray.
	^ [ array size ] bench! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 3/24/2005 00:09'!
initializeFixture

	session commit:
		[ session root
			at: 'benchmarkFixture' 
			put: (fixture := Dictionary new).
		"Force lazy-initialization; it was merely nice to have it with the getter rather than all piled up in one initializer."
		self 
			magmaCollectionFixture ;
			magmaArrayFixture ;
			oneMillionObjectPointersFixture ;
			oneThousandElementArrayFixture ;
			oneThousandElementArrayOfObjectsFixture ;
			singleObjectFixture ;
			thousandLevelDeepFixture ]! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 3/24/2005 00:25'!
magmaArrayFixture

	^ fixture
		at: #magmaArray
		ifAbsentPut: [ MagmaArray new at: 1000 put: -1; yourself ]! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmacollections' stamp: 'cmm 3/24/2005 00:07'!
magmaArrayTests

	^ #( getMagmaArraySize updateThousandsOfMagmaArrayElements )! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 3/13/2005 23:05'!
magmaCollectionFixture

	^ fixture
		at: #magmaCollectionFixture
		ifAbsentPut: [ MagmaCollection new ]! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmacollections' stamp: 'cmm 8/17/2005 22:25'!
magmaCollectionTests

	^ #( addThousandsOfObjectsTenAtATime addThousandsOfObjectsOneHundredAtATime addThousandsOfObjectsAtOnce )! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 12/8/2011 15:14'!
magmaPackageListDo: oneArgBlock 
	^ #('Ma-Core' 'Ma-Collections' 'Ma-Ascii-Report' 'Ma-Statistics' 'Ma-Serializer-Core' 'Ma-Client-Server-Core' 'Magma-Client' 'Magma-Server' 'Magma-Tools' 'Ma-Search' ) do: oneArgBlock! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 12/7/2011 15:50'!
oneMillionObjectPointersFixture
	^ fixture
		at: #oneMillionObjectPointersFixture
		ifAbsentPut:
			[ WbArray withAll:
				((1 to: 1000) collect:
					[ : x | WbArray new: 1000 ]) ]! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/15/2005 22:22'!
oneMillionObjectPointersRead

	^ self benchReadOf: (session oidFor: self oneMillionObjectPointersFixture)! !

!MagmaBenchmarker methodsFor: 'benchmarks-write' stamp: 'cmm 11/27/2011 11:40'!
oneThousandElementArrayCommit
	"How fast can it commit when there is a substantial buffer to write?"
	| count |
	count := 0.
	session abort; begin.
	self focusOn: #oneThousandElementArrayFixture.
	^ [ self oneThousandElementArrayFixture at: 1 put: (count := count + 1).
	session commitAndBegin ] bench! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 3/13/2005 23:06'!
oneThousandElementArrayFixture

	^ fixture
		at: #oneThousandElementArrayFixture
		ifAbsentPut: [ Array new: 1000 ]! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 12/7/2011 15:50'!
oneThousandElementArrayOfObjectsFixture
	^ fixture
		at: #oneThousandElementArrayOfObjectsFixture
		ifAbsentPut:
			[ WbArray withAll:
				((1 to: 1000) collect:
					[ : x | WbArray new: 1 ]) ]! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/15/2005 22:24'!
oneThousandElementArrayOfObjectsRead

	^ self benchReadOf: (session oidFor: self oneThousandElementArrayOfObjectsFixture)! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/15/2005 22:23'!
oneThousandElementArrayRead

	^ self benchReadOf: (session oidFor: self oneThousandElementArrayFixture)! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/15/2005 22:22'!
oneThousandLevelsDeepRead

	^ self benchReadOf: (session oidFor: self thousandLevelDeepFixture)! !

!MagmaBenchmarker methodsFor: 'benchmarks-write' stamp: 'cmm 11/27/2011 11:40'!
peakCommitRate
	"A minimal commit-package committed over and over to see what the fastest Magma's architecture can oscillate.  This is done using auto-begin to shave the last bit of efficiency."
	| count |
	count := 0.
	session abort; begin.
	self focusOn: #singleObjectFixture.
	^ [ self singleObjectFixture at: 1 put: (count := count +1).
	session commitAndBegin ] bench! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 10/20/2010 13:36'!
peakRefreshRate
	^ [ session refresh ] bench! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/24/2005 00:05'!
readTests

	^ #(peakRefreshRate singleObjectRead oneThousandElementArrayRead oneThousandElementArrayOfObjectsRead oneMillionObjectPointersRead oneThousandLevelsDeepRead)! !

!MagmaBenchmarker methodsFor: 'run' stamp: 'cmm 6/27/2008 00:24'!
run: selector resultsTo: aStream 
	aStream
		cr ;
		tab ;
		nextPutAll: selector ;
		nextPutAll: ' : ' ;
		nextPutAll: (self perform: selector).
	"Set each MaHistoricalStat's name to refer to this test, so we can see which test this stat is for."
	session statistics historysDo: 
		[ : eachStatHistory | 
		eachStatHistory isActive ifTrue: [ eachStatHistory active name: selector ] ].
	session statistics transition! !

!MagmaBenchmarker methodsFor: 'run' stamp: 'cmm 3/23/2005 22:38'!
runAll

	^ self runAll: self allCategories! !

!MagmaBenchmarker methodsFor: 'run' stamp: 'cmm 11/27/2011 15:13'!
runAll: categoriesArray 
	| answer priorShowStatusSetting |
	"MagmaPreferences showStatusViaCursor is turned on, which will dramatically reduce benchmark results."
	priorShowStatusSetting := MagmaPreferences showStatusViaCursor.
	MagmaPreferences showStatusViaCursor: false.
	answer := WriteStream on: String new.
	self
		 writeContextInformationTo: answer ;
		 initializeFixture.
	session readStrategy: MaReadStrategy deep.
	answer
		 cr ;
		 cr ;
		 nextPutAll: 'Benchmarks:'.
	Smalltalk garbageCollect.
	categoriesArray do:
		[ : each | self
			runCategory: each
			resultsTo: answer ].
	session disconnect.
	MagmaPreferences showStatusViaCursor: priorShowStatusSetting.
	^ answer contents! !

!MagmaBenchmarker methodsFor: 'run' stamp: 'cmm 3/13/2005 23:03'!
runCategory: selector resultsTo: aStream

	aStream
		cr; nextPutAll: '---';
		cr ;
		cr; nextPutAll: selector.
	(self perform: selector) do:
		[ : each |
		self
			run: each
			resultsTo: aStream ]! !

!MagmaBenchmarker methodsFor: 'accessing' stamp: 'cmm 6/10/2005 17:01'!
session
	^ session! !

!MagmaBenchmarker methodsFor: 'initialize-release' stamp: 'cmm 3/15/2005 22:21'!
session: aMagmaSession

	session := aMagmaSession! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 11/27/2011 11:37'!
singleObjectFixture
	^ fixture
		at: #singleObjectFixture
		ifAbsentPut: [ WordArray new: 1 ]! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 3/15/2005 22:22'!
singleObjectRead

	^ self benchReadOf: (session oidFor: self singleObjectFixture)! !

!MagmaBenchmarker methodsFor: 'fixtures' stamp: 'cmm 3/13/2005 23:06'!
thousandLevelDeepFixture

	^ fixture
		at: #thousandLevelDeepFixture
		ifAbsentPut:
			[ | rootOfDeepObject |
			rootOfDeepObject := Array new: 1.
			(1 to: 1000)
				inject: rootOfDeepObject
				into:
					[ : prior : x |
					prior at: 1 put: (Array new: 1) ].
			rootOfDeepObject ]! !

!MagmaBenchmarker methodsFor: 'accessing' stamp: 'cmm 3/23/2005 22:37'!
thousands
	"The size of the MagmaCollection tests."

	^ 1000! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 6/26/2008 22:59'!
timeToConnect
	"We do a manual #transition after every test, so allow up to 250 in the queue, none of which will be based on having transitioned due to time."
	session statistics snapshotInterval: (10 
			to: 2500
			by: 10).
	^ [ session connectAs: 'benchmarker' ] durationToRun printString! !

!MagmaBenchmarker methodsFor: 'benchmarks-read' stamp: 'cmm 1/11/2007 21:52'!
timeToDisconnect
	^ [ session disconnect ] durationToRun printString! !

!MagmaBenchmarker methodsFor: 'benchmarks-magmaarrays' stamp: 'cmm 11/14/2006 21:14'!
updateThousandsOfMagmaArrayElements
	| array |
	array := fixture at: #magmaArray.
	session begin.
	^ 
	[ 
	[ 1 
		to: self thousands
		do: 
			[ : x | 
			array 
				at: x
				put: x ].
	session commitAndBegin ] bench ] ensure: [ session abort ]! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 3/23/2005 23:47'!
writeBenchmarkerInformationTo: aStream

	aStream
		cr; cr;
		nextPutAll: 'Benchmarker Details:'.
	#(thousands)
		do:
			[ : each | 
			self
				writeEntry: each
				value: (self perform: each)
				to: aStream ]! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 9/22/2008 10:24'!
writeClientSessionInformationTo: aStream

	aStream
		cr; cr;
		nextPutAll: 'MagmaSession Details:'.
	#(isLocal)
		do:
			[ : each | 
			self
				writeEntry: each
				value: (session perform: each)
				to: aStream ].
	#(allowWriteBarrier)
		do:
			[ : each | 
			self
				writeEntry: each
				value: (session preferences perform: each)
				to: aStream ]! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 9/23/2008 14:54'!
writeContextInformationTo: aStream 
	aStream nextPutAll: 'The date is '.
	DateAndTime now printOn: aStream.
	self
		writeHardwareInformationTo: aStream ;
		writePlatformInformationTo: aStream ;
		writeVmInformationTo: aStream ;
		writeImageInformationTo: aStream ;
		writePackageInformationTo: aStream ;
		writeServerInformationTo: aStream ;
		writeClientSessionInformationTo: aStream ;
		writeBenchmarkerInformationTo: aStream! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 6/16/2005 23:43'!
writeEntry: selectorSymbol value: anObject to: aStream

	aStream
		cr; tab;
		nextPutAll: selectorSymbol withBlanksTrimmed ;
		nextPutAll: ' : ' ;
		maPrint: anObject! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 3/13/2005 18:14'!
writeHardwareInformationTo: aStream

	aStream
		cr ; cr ;
		nextPutAll: 'Hardware Details:'.
	(self class hardwareAttributes subStrings: ',') with: (self class hardwareInfo subStrings: ',') do:
		[ : eachAttribute : eachValue |
		self
			writeEntry: eachAttribute
			value: eachValue
			to: aStream ]! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 6/17/2009 10:12'!
writeImageInformationTo: aStream

	aStream
		cr; cr;
		nextPutAll: 'Image Details:'.
	self
		writeEntry: 'version'
		value: SystemVersion current version
		to: aStream.
	self
		writeEntry: 'lastUpdate'
		value: SystemVersion current highestUpdate printString
		to: aStream.
	self
		writeEntry: 'showStatusViaCursor'
		value: MagmaPreferences showStatusViaCursor
		to: aStream.
! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 6/26/2008 23:31'!
writePackageInformationTo: aStream 
	| workingCopy versionInformation |
	aStream
		cr ;
		cr ;
		nextPutAll: 'Code Package Details (from Monticello):'.
	self magmaPackageListDo: 
		[ : each | 
		workingCopy := (MCPackage named: each) workingCopy.
		versionInformation := workingCopy ancestors first.
		aStream
			cr ;
			cr ;
			nextPutAll: versionInformation summaryHeader.
		workingCopy needsSaving ifTrue: 
			[ aStream
				cr ;
				tab ;
				nextPutAll: '** modified **' ] ]! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 9/23/2008 14:54'!
writePlatformInformationTo: aStream 
	aStream
		cr ;
		cr ;
		nextPutAll: 'OS Details:'.
	#(#osVersion #platformName #platformSubtype ) do: 
		[ : each | 
		self 
			writeEntry: each
			value: (SmalltalkImage current perform: each)
			to: aStream ]! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 8/17/2005 23:01'!
writeServerInformationTo: aStream

	session isLocal ifFalse: [ ^ self "may want to figure out how to do this some day." ].
	aStream
		cr; cr;
		nextPutAll: 'Magma server details:'.
	#(flushAggressively)
		do:
			[ : each | 
			self
				writeEntry: each
				value: (session repositoryController perform: each)
				to: aStream ]! !

!MagmaBenchmarker methodsFor: 'benchmarks-write' stamp: 'cmm 11/27/2011 15:27'!
writeTests

	^ #(peakCommitRate oneThousandElementArrayCommit a1001BufferCommit aOneMillionObjectPointerCommit)! !

!MagmaBenchmarker methodsFor: 'capture context' stamp: 'cmm 9/11/2008 22:10'!
writeVmInformationTo: aStream

	aStream
		cr; cr;
		nextPutAll: 'Vm Details:'.
	#(vmVersion)
		do:
			[ : each | 
			self
				writeEntry: each
				value: (SmalltalkImage current perform: each)
				to: aStream ]! !

MaObject subclass: #MagmaBufferController
	instanceVariableNames: 'session primitiveAttributes commitLogSerializer bufferFacades commitRecords'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma-Tools-Buffer Editor'!

!MagmaBufferController commentStamp: 'cmm 3/15/2009 18:47' prior: 0!
This is a tool for browsing the actual MaObjectBuffers that make up each object of a Magma repoitory.

I help by presenting the physical domain-model as a graph of MagmaBufferFacades.  Each facade represents exactly one MaObjectBuffer, which is the primary unit-of-storage for a single object stored in Magma.  The facade provides as much information about the real object-model as possible, such as the class name or string contents if it is a String.

Should you be unable to open a repository you may build up my classMap as you uncover class-ids using clues from the buffers (for example, since all String buffers are automatically shown, you may recognize a reference to a particular String and identify the class-id of its referencer that way).

My 'cache' preserves facade identity, therefore allowing Maui to show the graph-shape with reference lines.

To use, you specify the location of the repository (or your own MagmaSession if you wish).
!

!MagmaBufferController class methodsFor: 'create' stamp: 'cmm 3/17/2010 16:54'!
location: aMagmaLocation 
	^ self session:
		(aMagmaLocation newSession
			allowWriteBarrier: MagmaPreferences canEnableWriteBarrier ;
			yourself)! !

!MagmaBufferController class methodsFor: 'maui' stamp: 'cmm 3/18/2009 13:48'!
mauiDefaultView
	^ 'Create'! !

!MagmaBufferController class methodsFor: 'create' stamp: 'cmm 3/14/2009 14:53'!
session: aMagmaSession
	^ self new
		session: aMagmaSession ;
		yourself! !

!MagmaBufferController methodsFor: 'root buffers' stamp: 'cmm 3/14/2009 14:57'!
anchorBuffer
	^ self bufferAt: (self primitiveAttributes at: #anchorOid)! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 3/15/2009 19:46'!
bufferAt: anInteger 
	^ bufferFacades 
		at: anInteger
		ifAbsent: [ self readBuffersAt: anInteger ]! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 3/23/2009 23:46'!
buffersInCommit: commitNumber 
	"The MaCommitLogEntry that contained this buffer."
	^ (self commitPackageAt: commitNumber) ifNotNilDo: 
		[ : cp | 
		| answer |
		answer := OrderedCollection new.
		cp objectsDo: 
			[ : eachBuffer | 
			answer add:
				(MagmaBufferFacade
					buffer: eachBuffer copyWithSameBuffer
					controller: self) ].
		answer ]! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 3/15/2009 19:45'!
cacheSize
	^ {  (bufferFacades size). (commitRecords size)  }! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 3/14/2009 14:43'!
classDefinitionFor: classIdInteger version: versionInteger 
	^ 
	[ self classIdManager 
		definitionForClassId: classIdInteger
		version: versionInteger ] 
		on: Error
		do: 
			[ : err | 
			(MaClassDefinition className: '(id=' , classIdInteger printString , ', v=' , versionInteger printString , ') unknown')
				id: classIdInteger ;
				version: versionInteger ;
				yourself ]! !

!MagmaBufferController methodsFor: 'root buffers' stamp: 'cmm 3/14/2009 15:01'!
classDefinitionsBuffer
	^ self bufferAt: (self primitiveAttributes at: #classDefinitionsOid)! !

!MagmaBufferController methodsFor: 'private' stamp: 'cmm 3/22/2005 23:25'!
classIdManager

	^ self serializer classIdManager! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 3/23/2009 23:40'!
commitLogEntryAt: commitNumber
	"The MaCommitLogEntry that contained this buffer."
	^ (self commitRecordAt: commitNumber) ifNotNilDo: [ : cr | self materializeCommitLogEntry: cr ]! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 4/14/2009 14:16'!
commitNumber
	^ session remoteCommitNumber! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 3/23/2009 23:41'!
commitPackageAt: commitNumber
	"The MaCommitLogEntry that contained this buffer."
	^ (self commitLogEntryAt: commitNumber) ifNotNilDo: [ : cle | cle commitPackage ]! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 3/15/2009 19:42'!
commitRecordAt: anInteger 
	^ commitRecords 
		at: anInteger
		ifAbsent: 
			[ 
			[ commitRecords 
				at: anInteger
				put: (self readCommitRecordAt: anInteger) ] 
				on: MagmaUnavailableCommitRecord
				do: [ : err | nil ] ]! !

!MagmaBufferController methodsFor: 'initializing' stamp: 'cmm 3/14/2009 15:29'!
disconnect
	session disconnect! !

!MagmaBufferController methodsFor: 'initializing' stamp: 'cmm 3/14/2009 15:30'!
disconnectAndClose
	session disconnectAndClose! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 3/15/2009 19:44'!
finalize
	commitRecords := commitRecords reject: [ : each | each isNil ].
	bufferFacades := bufferFacades reject: [ : each | each isNil ]! !

!MagmaBufferController methodsFor: 'initializing' stamp: 'cmm 3/15/2009 19:56'!
initialize
	super initialize.
	bufferFacades := Dictionary new.
	commitRecords := Dictionary new.
	commitLogSerializer := MagmaSession newCommitLogSerializer! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 4/14/2009 14:19'!
location
	^ session location! !

!MagmaBufferController methodsFor: 'private' stamp: 'cmm 3/14/2009 17:06'!
materializeCommitLogEntry: aMaCommitLogRecord 
	^ commitLogSerializer resetOidManager materializeGraph: aMaCommitLogRecord record! !

!MagmaBufferController methodsFor: 'maui' stamp: 'cmm 3/18/2009 13:48'!
mauiDefaultView
	^ 'Access'! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 3/14/2009 15:31'!
objectClassNameFor: classIdInteger 
	^ (self classIdManager 
		classDefinitionsForId: classIdInteger
		ifAbsent: [ ^ (classIdInteger , ' - unknown classId') asString ]) last name! !

!MagmaBufferController methodsFor: 'access' stamp: 'cmm 3/14/2009 15:00'!
primitiveAttributes
	^ primitiveAttributes ifNil: [ primitiveAttributes := session remotePrimitiveAttributes ]! !

!MagmaBufferController methodsFor: 'private' stamp: 'cmm 3/15/2009 19:46'!
readBuffersAt: oidInteger 
	| requestedBuffers |
	requestedBuffers := session rawBuffersFor: oidInteger.
	requestedBuffers do: 
		[ : each | 
		bufferFacades 
			at: each oid
			put: (MagmaBufferFacade 
					buffer: each
					controller: self) ].
	^ bufferFacades at: oidInteger! !

!MagmaBufferController methodsFor: 'private' stamp: 'cmm 3/15/2009 22:12'!
readCommitRecordAt: commitNumber 
	"Answer the MaCommitLogRecord for commitNumber."
	^ session commitLogRecordAt: commitNumber! !

!MagmaBufferController methodsFor: 'root buffers' stamp: 'cmm 3/14/2009 15:01'!
repositoryDefinitionBuffer
	^ self bufferAt: (self primitiveAttributes at: #definitionOid)! !

!MagmaBufferController methodsFor: 'private' stamp: 'cmm 3/14/2009 14:41'!
serializer
	^ session serializer! !

!MagmaBufferController methodsFor: 'initializing' stamp: 'cmm 3/14/2009 15:29'!
session: aMagmaSession 
	session := aMagmaSession.
	aMagmaSession user ifNil: [ aMagmaSession user: (MagmaUser id: self printString) ].
	aMagmaSession ensureConnected! !

MaObject subclass: #MagmaBufferFacade
	instanceVariableNames: 'buffer commitRecord controller annotation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma-Tools-Buffer Editor'!

!MagmaBufferFacade class methodsFor: 'create' stamp: 'cmm 3/22/2005 22:47'!
buffer: aMaObjectBuffer controller: aMagmaRepositoryController

	^ self new
		buffer: aMaObjectBuffer ;
		controller: aMagmaRepositoryController ;
		yourself! !

!MagmaBufferFacade methodsFor: 'initializing' stamp: 'cmm 3/22/2005 22:47'!
buffer: aMaObjectBuffer
	buffer := aMaObjectBuffer! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/14/2009 18:14'!
bufferAt: anInteger 
	"Answer the buffer I reference at slot anInteger."
	buffer isPointers ifFalse: [ ^ nil ].
	^ MaOidCalculator 
		objectWithOid: anInteger
		ifNone: [ controller bufferAt: anInteger ]! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/14/2009 17:35'!
bufferPrintString
	^ buffer isBytes 
		ifTrue: [ buffer asString ]
		ifFalse: [ 'a ' , self objectClassName ]! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/18/2009 10:12'!
buffersInSameCommit
	"All the buffers in my commitPackage."
	^ self commitPackage ifNotNilDo: 
		[ : cp | 
		| answer |
		answer := OrderedCollection new.
		cp objectsDo: 
			[ : eachBuffer | 
			answer add: (self class 
					buffer: eachBuffer copyWithSameBuffer
					controller: controller) ].
		answer ]! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 2/20/2009 18:03'!
classDefinition
	^ controller 
		classDefinitionFor: self classId
		version: self classVersion! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/22/2005 23:12'!
classId
	^ buffer classId! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/22/2005 23:13'!
classVersion
	^ buffer classVersion! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/14/2009 17:06'!
commitLogEntry
	"The MaCommitLogEntry that contained this buffer."
	^ self commitRecord ifNotNilDo: [ : cr | controller materializeCommitLogEntry: cr ]! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/14/2009 15:47'!
commitNumber
	^ buffer commitNumber! !

!MagmaBufferFacade methodsFor: 'private' stamp: 'cmm 3/14/2009 17:07'!
commitPackage
	"The MaCommitPackage that contained this buffer."
	^ self commitLogEntry ifNotNilDo: [ : cle | cle commitPackage ]! !

!MagmaBufferFacade methodsFor: 'private' stamp: 'cmm 3/23/2009 23:48'!
commitRecord
	"The MaCommitLogRecord that committed this buffer."
self maMarked: 'performance'.
"This is bad for an old db with no commit-log file history, it'll keep trying over and over... :("
	^ commitRecord ifNil: [ commitRecord := controller commitRecordAt: self commitNumber ]! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/14/2009 15:56'!
controller
	"Answer the MagmaBufferController for this buffer."
	^ controller! !

!MagmaBufferFacade methodsFor: 'initializing' stamp: 'cmm 3/23/2005 13:10'!
controller: aMagmaBufferController
	controller := aMagmaBufferController! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/14/2009 16:55'!
lastModified
	"The DateAndTime of the server the time this object was last committed."
	^ self commitRecord ifNotNilDo: [ : cr | cr timestamp ]! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/14/2009 17:35'!
mauiDefaultColumns
	^ #(#bufferPrintString #oid #lastModified #commitNumber )! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/14/2009 18:08'!
mauiName
	^ String streamContents: 
		[ : stream | 
		stream
			maPrint: self oid ;
			maPrint: ' - ' ;
			maPrint: self bufferPrintString ]! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/22/2005 23:24'!
objectClassName
	^ controller objectClassNameFor: self classId! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/14/2009 15:35'!
objectInstVarNames
	^ self classDefinition allInstVarNames ! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 3/22/2005 23:12'!
oid
	^ buffer oid! !

!MagmaBufferFacade methodsFor: 'printing' stamp: 'cmm 3/22/2005 23:40'!
printOn: aStream

	super printOn: aStream.
	buffer isBytes
		ifTrue:
			[ aStream nextPutAll: buffer asString ]
		ifFalse:
			[ self
				maPrintAttributes: #(oid objectClassName)
				on: aStream ]! !

!MagmaBufferFacade methodsFor: 'access' stamp: 'cmm 8/27/2011 11:32'!
referencedBuffers
	^ buffer isPointers ifTrue:
		[ (1 to: buffer objectInstSize) collect:
			[ : index | self bufferAt: (buffer maInstVarAt: index) ] ]! !

MaObject subclass: #MagmaDoctor
	instanceVariableNames: 'filer invalidClassDefinitionsBuffer invalidDefinitionBuffer invalidAnchorBuffer classIdOfDictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma-Tools-Health'!

!MagmaDoctor commentStamp: 'cmm 3/25/2007 19:05' prior: 0!
I present a report on a MagmaRepository.  I open the repository specified, report all of its primitive-attributes, then the contents and state of the crucial buffers (classDefinitions, anchor, definition).  

If any of them do not look correct, I perform a scan of the object file, looking for them.
!

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:19'!
dataAttributes
	^ { }! !

!MagmaDoctor methodsFor: 'private' stamp: 'cmm 5/11/2007 14:40'!
filer
	^ filer! !

!MagmaDoctor methodsFor: 'initialize-release' stamp: 'cmm 3/25/2007 19:18'!
filer: aMaObjectFiler
	filer := aMaObjectFiler! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:16'!
headerAttributes
	^ {  (TextEmphasis bold). (TextEmphasis underlined)  }! !

!MagmaDoctor methodsFor: 'initialize-release' stamp: 'cmm 3/29/2007 15:21'!
initialize
	super initialize.
	invalidAnchorBuffer := invalidClassDefinitionsBuffer := invalidDefinitionBuffer := false! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:29'!
invalid
	^ '**INVALID**'! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:18'!
labelAttributes
	^ {  (TextEmphasis bold)  }! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:18'!
looksLikeAnchorBuffer: aMaObjectBuffer 
	| stream |
	stream := WriteStream on: String new.
	self 
		printProblemsWithAnchorBuffer: aMaObjectBuffer
		on: stream.
	^ stream isEmpty! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/26/2007 23:01'!
looksLikeClassDefinitionBuffer: aMaObjectBuffer
	| stream |
	stream := WriteStream on: String new.
	self printProblemsWithClassDefinitionsBuffer: aMaObjectBuffer on: stream.
	^ stream isEmpty! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:13'!
looksLikeDefinitionBuffer: aMaObjectBuffer 
	| stream |
	stream := WriteStream on: String new.
	self 
		printProblemsWithDefinitionBuffer: aMaObjectBuffer
		on: stream.
	^ stream isEmpty! !

!MagmaDoctor methodsFor: 'accessing' stamp: 'cmm 3/25/2007 19:19'!
openHealthReport
	| ts |
	ts := TextStream on: Text new.
	self writeHealthInformationTo: ts.
	(Workspace new
		contents: ts contents ;
		yourself) openLabel: self pathName! !

!MagmaDoctor methodsFor: 'accessing' stamp: 'cmm 5/11/2007 13:10'!
pathName
	^ filer directory pathName! !

!MagmaDoctor methodsFor: 'private' stamp: 'cmm 3/29/2007 15:14'!
potentialAnchorBuffers
	| potentialAnswers |
	potentialAnswers := OrderedCollection new.
	self filer objectBuffersDo: 
		[ : each | 
		(self looksLikeAnchorBuffer: each) ifTrue: [ potentialAnswers add: each ] ].
	^ potentialAnswers! !

!MagmaDoctor methodsFor: 'private' stamp: 'cmm 3/29/2007 15:38'!
potentialClassDefinitionBuffers
	| potentialAnswers |
	potentialAnswers := OrderedCollection new.
	self filer objectBuffersDo: 
		[ : each | 
		(self looksLikeClassDefinitionBuffer: each) ifTrue: [ potentialAnswers add: each ] ].
	(potentialAnswers collect: [ : e | e classId ]) asSet in: [ : classIds | classIds size = 1 ifTrue: [ classIdOfDictionary := classIds anyOne ] ].
	^ potentialAnswers! !

!MagmaDoctor methodsFor: 'private' stamp: 'cmm 3/29/2007 15:14'!
potentialDefinitionBuffers
	| potentialAnswers |
	potentialAnswers := OrderedCollection new.
	self filer objectBuffersDo: 
		[ : each | 
		(self looksLikeDefinitionBuffer: each) ifTrue: [ potentialAnswers add: each ] ].
	^ potentialAnswers! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:46'!
printAnchorBufferOn: aWriteStream 
	| buffer |
	self 
		printHeading: 'The Anchor Buffer'
		on: aWriteStream.
	buffer := self filer bufferFor: self filer anchorOid.
	self 
		printFieldNamed: 'oid'
		value: buffer oid
		on: aWriteStream.
	self 
		printFieldNamed: 'bufferType'
		value: buffer bufferType
		on: aWriteStream.
	self 
		printFieldNamed: #objectInstSize
		value: buffer objectInstSize
		on: aWriteStream.
	self 
		printFieldNamed: #classId
		value: buffer classId
		on: aWriteStream.
	self 
		printProblemsWithAnchorBuffer: buffer
		on: aWriteStream.
"	invalidAnchorBuffer ifTrue: [ self printPotentialAnchorBuffersOn: aWriteStream ]"! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:12'!
printClassDefinitionsBufferOn: aWriteStream 
	| buffer |
	self 
		printHeading: 'The Class-Definitions Buffer'
		on: aWriteStream.
	buffer := self filer bufferFor: self filer classDefinitionsOid.
	self 
		printFieldNamed: 'oid'
		value: buffer oid
		on: aWriteStream.
	self 
		printFieldNamed: 'bufferType'
		value: buffer bufferType
		on: aWriteStream.
	self 
		printFieldNamed: #objectInstSize
		value: buffer objectInstSize
		on: aWriteStream.
	self 
		printFieldNamed: #classId
		value: buffer classId
		on: aWriteStream.
	self 
		printProblemsWithClassDefinitionsBuffer: buffer
		on: aWriteStream.
	invalidClassDefinitionsBuffer ifTrue: [ self printPotentialClassDefinitionsBuffersOn: aWriteStream ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 19:04'!
printCrucialBuffersOn: aWriteStream 
	self
		printClassDefinitionsBufferOn: aWriteStream ;
		printDefinitionBufferOn: aWriteStream ;
		printAnchorBufferOn: aWriteStream! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:49'!
printDefinitionBufferOn: aWriteStream 
	| buffer |
	self 
		printHeading: 'The Repository-Definition Buffer'
		on: aWriteStream.
	buffer := self filer bufferFor: self filer definitionOid.
	self 
		printFieldNamed: 'oid'
		value: buffer oid
		on: aWriteStream.
	self 
		printFieldNamed: 'bufferType'
		value: buffer bufferType
		on: aWriteStream.
	self 
		printFieldNamed: #objectInstSize
		value: buffer objectInstSize
		on: aWriteStream.
	self 
		printFieldNamed: #classId
		value: buffer classId
		on: aWriteStream.
	self 
		printProblemsWithDefinitionBuffer: buffer
		on: aWriteStream.
	"invalidDefinitionBuffer ifTrue: [ self printPotentialDefinitionBuffersOn: aWriteStream ]"! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/26/2007 22:56'!
printFieldNamed: aString value: anObject on: aWriteStream 
	self 
		printLabel: aString
		value: anObject
		withAttributes: self dataAttributes
		on: aWriteStream! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 19:24'!
printHeading: aString on: aWriteStream 
	aWriteStream
		cr ;
		cr ;
		withAttributes: self headerAttributes
			do: [ aWriteStream maPrint: aString ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/26/2007 22:57'!
printInvalidOn: aWriteStream withMessage: aString 
	self 
		printLabel: self invalid
		value: aString
		withAttributes: self problemAttributes
		on: aWriteStream! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/26/2007 22:56'!
printLabel: aString value: anObject withAttributes: attributes on: aWriteStream 
	aWriteStream 
		withAttributes: attributes
		do: 
			[ aWriteStream
				cr ;
				tab ;
				maPrint: aString ;
				maPrint: ':  ' ].
	aWriteStream 
		withAttributes: self dataAttributes
		do: [ aWriteStream maPrint: anObject ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:28'!
printPotentialAnchorBuffersOn: aWriteStream 
	self 
		printHeading: 'Candidates for the real AnchorBuffer'
		on: aWriteStream.
	self potentialAnchorBuffers do: 
		[ : each | 
		aWriteStream
			cr ;
			tab ;
			maPrint: each ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:36'!
printPotentialClassDefinitionsBuffersOn: aWriteStream 
	self 
		printHeading: 'Candidates for the real ClassDefinitions buffer'
		on: aWriteStream.
	self potentialClassDefinitionBuffers do: 
		[ : each | 
		aWriteStream
			cr ;
			tab ;
			maPrint: each ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:36'!
printPotentialDefinitionBuffersOn: aWriteStream 
	self 
		printHeading: 'Candidates for the real Definition buffer'
		on: aWriteStream.
	self potentialDefinitionBuffers do: 
		[ : each | 
		aWriteStream
			cr ;
			tab ;
			maPrint: each ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:51'!
printPrimitiveAttributesOn: aWriteStream 
	self 
		printHeading: 'Primitive Repository Values'
		on: aWriteStream.
	self
		printSignatureOn: aWriteStream ;
		printVersionOn: aWriteStream.
	#(#classDefinitionsOid #definitionOid #anchorOid ) do: 
		[ : each | 
		self 
			printSpecialOid: each
			on: aWriteStream ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:20'!
printProblemsWithAnchorBuffer: aMaObjectBuffer on: aWriteStream 
	aMaObjectBuffer bufferType = MaFixedObjectBuffer basicNew bufferType ifFalse: 
		[ invalidAnchorBuffer := true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'The buffer should be a MaFixedObjectBuffer, it isn''t.' ].
	aMaObjectBuffer objectInstSize = MaRootAnchor instSize ifFalse: 
		[ invalidAnchorBuffer := true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'This buffer should represent a MaRootAnchor, but its instSize isn''t right for that' ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:08'!
printProblemsWithClassDefinitionsBuffer: aMaObjectBuffer on: aWriteStream 
	| indexOfNonIntegerClassId indexOfInvalidUserOidClassDef |
	aMaObjectBuffer bufferType = MaVariableObjectBuffer basicNew bufferType ifFalse: 
		[ invalidClassDefinitionsBuffer := true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'The buffer should be a MaVariableObjectBuffer, it isn''t.'.
		^ self ].
	(aMaObjectBuffer objectInstSize even and: [ aMaObjectBuffer objectInstSize > 0 ]) ifFalse: 
		[ invalidClassDefinitionsBuffer := true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'This buffer should represent a Dictionary, so it should always have an even objectInstSize (key->value pairs).'.
		^ self ].
	indexOfNonIntegerClassId := 0.
	indexOfInvalidUserOidClassDef := 0.
	aMaObjectBuffer instVarsDoWithIndex: 
		[ : eachOid : index | 
		index odd 
			ifTrue: 
				[ (MaOidCalculator isOidForOptimizedInteger: eachOid) ifFalse: [ indexOfNonIntegerClassId := index ] ]
			ifFalse: 
				[ (MaOidCalculator isOidForUserObject: eachOid) ifFalse: [ indexOfInvalidUserOidClassDef := index ] ] ].
	indexOfNonIntegerClassId = 0 ifFalse: 
		[ invalidClassDefinitionsBuffer := true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'The class-definitions buffer should be a Dictionary with SmallIntegers as its keys.  This is not the case at instVar' , indexOfNonIntegerClassId printString ].
	indexOfInvalidUserOidClassDef = 0 ifFalse: 
		[ invalidClassDefinitionsBuffer := true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'The class-definitions buffer should be a Dictionary user objects (MaClassDefinitions) as its values.  This is not the case at instVar' , indexOfInvalidUserOidClassDef printString ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/29/2007 15:12'!
printProblemsWithDefinitionBuffer: aMaObjectBuffer on: aWriteStream 
	aMaObjectBuffer bufferType = MaFixedObjectBuffer basicNew bufferType ifFalse: 
		[ invalidDefinitionBuffer := true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'The buffer should be a MaFixedObjectBuffer, it isn''t.' ].
	aMaObjectBuffer objectInstSize = MagmaRepositoryDefinition instSize ifFalse: 
		[ invalidDefinitionBuffer := true.
		self 
			printInvalidOn: aWriteStream
			withMessage: 'This buffer should represent a MagmaRepositoryDefinition, but its instSize isn''t right for that' ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 13:31'!
printRepositoryPathOn: aWriteStream 
	aWriteStream
		cr ;
		maPrint: 'pathName:  ' ;
		maPrint: self pathName! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 19:21'!
printSignatureOn: aWriteStream 
	self 
		printFieldNamed: 'signature'
		value: self filer signature
		on: aWriteStream.
	[ self filer validateSignature ] 
		on: MagmaUserError
		do: 
			[ : err | 
			self 
				printInvalidOn: aWriteStream
				withMessage: err messageText ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 19:31'!
printSpecialOid: selectorSymbol on: aWriteStream 
	self 
		printFieldNamed: selectorSymbol
		value: (self filer perform: selectorSymbol)
		on: aWriteStream.
	[ self filer validateSpecialOid: selectorSymbol ] 
		on: MagmaCorruptionError
		do: 
			[ : err | 
			self 
				printInvalidOn: aWriteStream
				withMessage: err messageText ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:53'!
printTitleOn: aWriteStream 
	aWriteStream 
		withAttributes: { 
				(TextAlignment centered).
				(TextEmphasis underlined)
			 }
		do: [ aWriteStream maPrint: self pathName ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 19:21'!
printVersionOn: aWriteStream 
	self 
		printFieldNamed: 'version'
		value: self filer version
		on: aWriteStream.
	[ self filer validateVersion ] 
		on: MagmaUserError
		do: 
			[ : err | 
			self 
				printInvalidOn: aWriteStream
				withMessage: err messageText ]! !

!MagmaDoctor methodsFor: 'private-report generation' stamp: 'cmm 3/25/2007 14:20'!
problemAttributes
	^ { TextColor red.  TextEmphasis bold }! !

!MagmaDoctor methodsFor: 'accessing' stamp: 'cmm 3/25/2007 14:25'!
writeHealthInformationTo: aWriteStream
	self
		printTitleOn: aWriteStream ;
		printPrimitiveAttributesOn: aWriteStream ;
		printCrucialBuffersOn: aWriteStream! !

MaObject subclass: #MagmaStressTest
	instanceVariableNames: 'config session uuid collection counter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma-Tools-Stress Tester'!

!MagmaStressTest class methodsFor: 'run' stamp: 'cmm 12/21/2008 15:16'!
clientStressorImageName
	^ '_stressTestClient'! !

!MagmaStressTest class methodsFor: 'run' stamp: 'cmm 12/19/2008 15:03'!
createFrom: configurations at: aMagmaRemoteLocation 
	"Collect MagmaSessions connected to aMagmaRemoteLocation for each instance of configurations.  An inspector to a connected stressor is opened for each configuration.  They must be told to flood separately."
	^ configurations collect: 
		[ : each | 
		| sess stressor |
		sess := aMagmaRemoteLocation newSession user: (MagmaUser id: each printString).
		"Minute-by-minute for the last week."
		sess statistics snapshotInterval: (1 to: 60 * 24 * 7 by: 1).
		stressor := self 
			session: sess
			configuration: each.
		stressor inspect ]
! !

!MagmaStressTest class methodsFor: 'initialize' stamp: 'cmm 1/2/2009 15:41'!
createRepository
	MagmaRepositoryController 
		create: self dbName, self port asString
		root: (Dictionary new
				at: #collection
					put: self newFixture ;
				yourself)! !

!MagmaStressTest class methodsFor: 'initialize' stamp: 'cmm 12/17/2008 10:52'!
dbName
	^ 'stressTest'! !

!MagmaStressTest class methodsFor: 'initialize' stamp: 'cmm 1/2/2009 15:41'!
deleteRepository
	MagmaRepositoryController delete: self dbName, self port asString! !

!MagmaStressTest class methodsFor: 'private' stamp: 'cmm 5/13/2010 20:42'!
fullScreenOff
	(Smalltalk classNamed: #ScreenController)
		ifNotNilDo: [:sc | sc new fullScreenOff].
	"Easier window management in the OS by sizing the windows down.."
	(Smalltalk classNamed: #Display)
		ifNotNilDo: 
			[ : disp | 
			disp class
				depth: Display depth
				width: Display width * 3 // 4
				height: Display height * 3 // 4
				fullscreen: false].
	(Smalltalk classNamed: #World)
		ifNotNilDo: [:world | world doOneCycleNow]! !

!MagmaStressTest class methodsFor: 'initialize' stamp: 'cmm 1/2/2009 15:57'!
newFixture
	^ MagmaCollection new
		addIndex: (MaIntegerIndex attribute: #key) ;
		yourself! !

!MagmaStressTest class methodsFor: 'accessing' stamp: 'cmm 12/17/2008 14:37'!
port
	^ 51801! !

!MagmaStressTest class methodsFor: 'run' stamp: 'cmm 12/17/2008 20:08'!
prepareStressors: configurations at: aMagmaRemoteLocation andSaveClientStressorAs: newImageName 
	"Create new MagmaStressTestors based on configurations, but save-and-exit the image before flooding the server.  This allows this saved image to simply be started on any machine to add to the server stress."
	| stressors |
	stressors := self 
		createFrom: configurations
		at: aMagmaRemoteLocation.
	self saveAndExitAs: newImageName.
	stressors do: [ : each | [ each floodServerUntil: [ false ] ] fork ]! !

!MagmaStressTest class methodsFor: 'initialize' stamp: 'cmm 12/17/2008 13:19'!
resetRepository
	
	[ self
		deleteRepository ;
		createRepository ] 
		on: MagmaDeleteWarning
		do: [ : warn | warn resume ]! !

!MagmaStressTest class methodsFor: 'private' stamp: 'cmm 4/11/2010 18:10'!
saveAndExitAs: imageName 
	"Save the image under the name, imageName."
	self fullScreenOff.
	SmalltalkImage current maSaveAs: (SmalltalkImage current fullNameForImageNamed: imageName).
	SmalltalkImage current 
		snapshot: true
		andQuit: true! !

!MagmaStressTest class methodsFor: 'run' stamp: 'cmm 12/18/2008 18:29'!
saveAsServerImageAndExit
	self saveAsServerImageAndExitPort: self port ! !

!MagmaStressTest class methodsFor: 'run' stamp: 'cmm 3/18/2010 18:57'!
saveAsServerImageAndExitPort: portInteger 
	| console |
	self saveAndExitAs: '_stressTestServer' , portInteger asString.
	console := MagmaServerConsole new.
	"Capture stats minute-by-minute for the last week."
	console statistics snapshotInterval: (1 to: 60 * 24 * 7 by: 1).
	console
		open: MagmaStressTest dbName , portInteger asString ;
		processOn: portInteger ;
		useCompression: false.
	console controller flushAggressively: false.
	portInteger = self port ifTrue: 
		[ (Delay forSeconds: 6) wait. console controller fullBackupToServerPath: 'stressTest' , (portInteger + 1) asString ].
	"Reference globally to eliminate chance of garbage-collect."
	console inspect! !

!MagmaStressTest class methodsFor: 'run' stamp: 'cmm 1/2/2009 15:18'!
saveClientStressorImageAndExitConnectingTo: hostName 
self saveClientStressorImageAndExitConnectingTo: hostName port: self port! !

!MagmaStressTest class methodsFor: 'run' stamp: 'cmm 1/2/2009 15:17'!
saveClientStressorImageAndExitConnectingTo: hostName port: portInteger
	| conf loc |
	conf := MagmaStressTestConfiguration new
		objectsPerCommit: 10 ;
		yourself.
	loc := MagmaRemoteLocation 
		host: hostName
		port: portInteger.
	MagmaStressTest 
		prepareStressors: {  conf  }
		at: loc
		andSaveClientStressorAs: self clientStressorImageName! !

!MagmaStressTest class methodsFor: 'create' stamp: 'cmm 12/17/2008 13:21'!
session: aMagmaSession configuration: aMagmaStressTestConfiguration 
	^ self new
		session: aMagmaSession ;
		configuration: aMagmaStressTestConfiguration ;
		yourself! !

!MagmaStressTest methodsFor: 'private' stamp: 'cmm 12/17/2008 13:17'!
commitAnother
	"Submit another entry to the collection."
	collection add: self newEntry.
	session commitAndBegin! !

!MagmaStressTest methodsFor: 'access' stamp: 'cmm 12/17/2008 13:04'!
configuration
	"Answer theMagmaStressTestConfiguration used to guide the receiver."
	^ config! !

!MagmaStressTest methodsFor: 'initialize-release' stamp: 'cmm 12/17/2008 14:42'!
configuration: aMagmaStressTestConfiguration 
	config := aMagmaStressTestConfiguration! !

!MagmaStressTest methodsFor: 'access' stamp: 'cmm 12/17/2008 20:43'!
floodServerUntil: untilBlock 
	collection ifNil: [ collection := session root at: #collection ].
	session inTransaction ifFalse: [ session begin ].
	untilBlock whileFalse: [ self commitAnother ]! !

!MagmaStressTest methodsFor: 'private' stamp: 'cmm 1/2/2009 15:59'!
incrementCounter
	^ counter := counter + 1! !

!MagmaStressTest methodsFor: 'initialize-release' stamp: 'cmm 1/2/2009 15:59'!
initialize
	super initialize.
	config := MagmaStressTestConfiguration new.
	uuid := UUID new.
	counter := 0! !

!MagmaStressTest methodsFor: 'private' stamp: 'cmm 1/2/2009 15:59'!
newEntry
	^ Association 
		key: self incrementCounter
		value: {  uuid. (self newObjectArray)  }! !

!MagmaStressTest methodsFor: 'private' stamp: 'cmm 12/17/2008 12:58'!
newObjectArray
	^ (1 to: config objectsPerCommit) collect: [ : each | Object new ]! !

!MagmaStressTest methodsFor: 'private' stamp: 'cmm 12/17/2008 12:51'!
session
	"The MagmaSession used to wail on the server."
	^ session! !

!MagmaStressTest methodsFor: 'initialize-release' stamp: 'cmm 12/17/2008 20:44'!
session: aMagmaSession 
	session := aMagmaSession! !

MaObject subclass: #MagmaStressTestConfiguration
	instanceVariableNames: 'objectsPerCommit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma-Tools-Stress Tester'!

!MagmaStressTestConfiguration methodsFor: 'initialize-release' stamp: 'cmm 12/17/2008 12:43'!
initialize
	super initialize.
	objectsPerCommit := 1! !

!MagmaStressTestConfiguration methodsFor: 'accessing' stamp: 'cmm 12/17/2008 12:41'!
objectsPerCommit
	^objectsPerCommit! !

!MagmaStressTestConfiguration methodsFor: 'accessing' stamp: 'cmm 12/17/2008 12:41'!
objectsPerCommit: anInteger
	objectsPerCommit := anInteger! !

!MagmaStressTestConfiguration methodsFor: 'printing' stamp: 'cmm 12/17/2008 13:27'!
printOn: aStream 
	self 
		maPrintAttribute: #objectsPerCommit
		on: aStream! !

MaObject subclass: #MagmaUpgrader
	instanceVariableNames: 'location definition session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma-Tools-Upgrade'!

MagmaUpgrader subclass: #Magma15To16Upgrader
	instanceVariableNames: 'classIdsToDisconnect stillConnected classIdForSet classIdForDictionary classIdsForSetsAndDictionarys'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma-Tools-Upgrade'!

!Magma15To16Upgrader commentStamp: 'cmm 8/5/2011 16:20' prior: 0!
Upgrading from Magma version 15 to 16:
	- First, create an admin-session to the legacy repository and run #fullBackup.
	- Now, still using the legacy version (15) of Magma, load in the latest MagmaTools, which includes this code.
	- Write a script Magma15To16Upgrader to disconnect all BlockContexts and CompiledMethods from your model.  Most repositories won't have any.  But, if so, write a script to convert them to MessageSends or whatever.  You should now be ready for the upgrade step.

	- #verifyCanUpgrade.  Identify all Block and ContextPart buffers.
	- #upgradeBuffers.  Identify all buffers whose #type is 2, a Variable buffer, but which are NOT a Set or Dictionary (ck #classId).  These buffers need to have their #type: changed to Fixed (3) and their classVersion: set to 1.
	- Open the converted repository in a new image with the Magma 16 code.!

!Magma15To16Upgrader methodsFor: 'private' stamp: 'cmm 8/9/2011 13:05'!
classIdsForSetsAndDictionarys
	^ classIdsForSetsAndDictionarys ifNil:
		[ classIdsForSetsAndDictionarys := Array streamContents:
			[ : stream | {Dictionary. 
			Set} do:
				[ : eachRootClass | eachRootClass withAllSubclassesDo:
					[ : eachClass | (self session serializer classIdManager includesIdForClass: eachClass) ifTrue: [ stream nextPut: (self session serializer classIdManager idForClass: eachClass) ] ] ] ] ]! !

!Magma15To16Upgrader methodsFor: 'accessing' stamp: 'cmm 8/5/2011 15:36'!
classIdsToDisconnect
	^ classIdsToDisconnect ifNil:
		[ classIdsToDisconnect := Array streamContents:
			[ : stream | | classIdManager |
			classIdManager := self session serializer classIdManager.
			#(#BlockClosure #BlockContext #MethodContext #CompiledMethod #MaBlockContextStorage #MaMethodContextStorage #MaCompiledMethodStorage ) do:
				[ : each | (classIdManager hasClassDefinitionFor: (Smalltalk classNamed: each)) ifTrue: [ stream nextPut: (classIdManager idForClass: (Smalltalk classNamed: each)) ] ] ] ]! !

!Magma15To16Upgrader methodsFor: 'private' stamp: 'cmm 8/5/2011 16:39'!
legacyVersion
	^ 15! !

!Magma15To16Upgrader methodsFor: 'private' stamp: 'cmm 8/5/2011 14:08'!
postUpgradeModel
	MagmaSession version = 16 ifFalse: [ MagmaUserError signal: 'This step should only be run after you have loaded Magma version 16 code.' ]! !

!Magma15To16Upgrader methodsFor: 'private' stamp: 'cmm 8/7/2011 21:02'!
prepareModel
	self halt: 'Override this method to disconnect all BlockClosures and ContextParts (incl. instances of all subclasses) and CompiledMethods from your model.  Most repositories won''t have any.  But, if so, write a script to convert them to MessageSends or whatever.
	The next step will traverse the entire object model to make sure there are no more instances of any of those classes.'! !

!Magma15To16Upgrader methodsFor: 'private' stamp: 'cmm 8/7/2011 14:56'!
restoreModel
	"You may wish to put BlockClosures back."! !

!Magma15To16Upgrader methodsFor: 'private' stamp: 'cmm 8/5/2011 15:37'!
stillConnected
	"The oids of CompiledMethods, MethodContexts, BlockContexts, BlockClosures, or storage for any of those."
	^ stillConnected ifNil: [ stillConnected := OrderedCollection new ]! !

!Magma15To16Upgrader methodsFor: 'accessing' stamp: 'cmm 8/5/2011 15:44'!
traceStillConnected
	"Answer all paths to CompiledMethods and ContextParts."
	^ (MagmaFileTraverser for: location) trace: self stillConnected! !

!Magma15To16Upgrader methodsFor: 'private' stamp: 'cmm 8/9/2011 22:12'!
upgradeBuffers
	"Identify all classId's for buffers whose #type is 2, a Variable buffer, but which are NOT a Set or Dictionary (ck #classId).  These buffers need to have their #type: changed to Fixed (3) and their classVersion: set to 1."
	| compressor |
	MagmaNotification signal: 'Upgrading buffers...'.
	compressor := (MagmaCompressor source: location) upgradeBlock:
		[ : eachBuffer | (eachBuffer type = 2 and: [ (self classIdsForSetsAndDictionarys includes: eachBuffer classId) not ]) ifTrue:
			[ eachBuffer type: 3.
			eachBuffer asSentientBuffer classVersion: 1 ].
		eachBuffer ].
	compressor compressTo: (MagmaLocalLocation path: location path , '.v16')! !

!Magma15To16Upgrader methodsFor: 'accessing' stamp: 'cmm 8/7/2011 20:59'!
verifyCanUpgrade
	"Make sure there are no buffers for CompiledMethods, MethodContexts, BlockContext or BlockClosures."
	self classIdsToDisconnect ifNotEmpty:
		[ [ | traverser | (traverser := MagmaFileTraverser for: location) anchorGraphDo:
			[ : each | (self classIdsToDisconnect includes: each classId) ifTrue:
				[ self stillConnected add:
					(traverser graphBuffer buffers collect:
						[ : eachBuffer | (self session objectWithOid: eachBuffer oid)-> eachBuffer ]) ] ] ]
			on: MaNotification
			do:
				[ : noti | Transcript
					 cr ;
					 show: noti messageText.
				noti resume ] ].
	self stillConnected ifNotEmpty:
		[ self close.
		self stillConnected explore.
		MagmaUserError signal: 'This repository still has these references to CompiledMethods, MethodContexts, BlockContext or BlockClosures.  They must be disconnected before it will be safe to perform the upgrade.' ]! !

!Magma15To16Upgrader methodsFor: 'private' stamp: 'cmm 8/5/2011 14:08'!
verifyRunning16
	MagmaSession version = 16 ifFalse: [ MagmaUserError signal: 'This step should only be run after you have loaded Magma version 16 code.' ]! !

!Magma15To16Upgrader methodsFor: 'private' stamp: 'cmm 8/6/2011 17:41'!
verifyRunningLegacyVersion
	MagmaSession magmaVersion = 15 ifFalse: [ MagmaUserError signal: 'This step should only be run with the legacy Magma version 15 code.' ]! !

!Magma15To16Upgrader methodsFor: 'private' stamp: 'cmm 8/6/2011 17:41'!
warn
	Warning signal: 'About to upgrade the local repository at ', location path, ' from version 15 to 16.  Be sure you have made a backup of this repository!!'! !

!MagmaUpgrader class methodsFor: 'instance creation' stamp: 'cmm 8/5/2011 14:00'!
location: aMagmaLocation 
	^ self new
		 setLocation: aMagmaLocation ;
		 yourself! !

!MagmaUpgrader methodsFor: 'initialize-release' stamp: 'cmm 8/5/2011 16:28'!
close
	session ifNotNil: [ session disconnectAndClose ]! !

!MagmaUpgrader methodsFor: 'accessing' stamp: 'cmm 8/5/2011 14:19'!
definition
	"Answer the repository definition."
	^ definition ifNil: [ definition := self session definition ]! !

!MagmaUpgrader methodsFor: 'private' stamp: 'cmm 8/5/2011 16:34'!
legacyVersion
	self subclassResponsibility ! !

!MagmaUpgrader methodsFor: 'private' stamp: 'cmm 8/7/2011 18:30'!
newVersion
	^ self legacyVersion+1! !

!MagmaUpgrader methodsFor: 'private' stamp: 'cmm 8/7/2011 20:02'!
newVersionLocation
	^ MagmaLocalLocation path: location path , '.v', self newVersion asString! !

!MagmaUpgrader methodsFor: 'upgrade' stamp: 'cmm 8/5/2011 14:10'!
postUpgrade
	"Code to run with new version of Magma."
	self postUpgradeModel! !

!MagmaUpgrader methodsFor: 'private' stamp: 'cmm 8/5/2011 14:14'!
prepareModel
	self subclassResponsibility! !

!MagmaUpgrader methodsFor: 'accessing' stamp: 'cmm 8/7/2011 21:29'!
session
	^ session ifNil:
		[ session := location newSession
			 userId: 'upgrader from 15 to 16' ;
			 ensureConnected ;
			 yourself ]! !

!MagmaUpgrader methodsFor: 'initialize-release' stamp: 'cmm 8/5/2011 14:00'!
setLocation: aMagmaLocation
	location := aMagmaLocation! !

!MagmaUpgrader methodsFor: 'private' stamp: 'cmm 8/7/2011 18:30'!
setVersion
	| filer |
	MagmaNotification signal: 'Setting version to ', self newVersion.
	filer := MaObjectFiler open: self newVersionLocation directory.
	filer version: self legacyVersion + 1.
	filer close! !

!MagmaUpgrader methodsFor: 'upgrade' stamp: 'cmm 8/7/2011 18:29'!
upgrade
	[ [ self
		 verifyRunningLegacyVersion ;
		 warn ;
		 prepareModel ;
		 verifyCanUpgrade ;
		 upgradeBuffers ;
		 setVersion.
		 MagmaNotification signal: 'Upgrade complete.' ] ensure: [ self close ] ]
		on: MaNotification
		do:
			[ : noti | Transcript
				 cr ;
				 show: noti messageText.
			noti resume ]! !

!MagmaUpgrader methodsFor: 'private' stamp: 'cmm 8/5/2011 14:14'!
upgradeBuffers
	self subclassResponsibility! !

!MagmaUpgrader methodsFor: 'private' stamp: 'cmm 8/5/2011 16:34'!
verifyRunningLegacyVersion
	MagmaSession magmaVersion = self legacyVersion! !

!MagmaUpgrader methodsFor: 'private' stamp: 'cmm 8/5/2011 14:00'!
warn
	self subclassResponsibility! !

PackageInfo subclass: #MagmaToolsPackageInfo
	instanceVariableNames: 'console'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma-Tools-Info'!

!MagmaToolsPackageInfo class methodsFor: 'as yet unclassified' stamp: 'cmm 4/21/2011 16:47'!
initialize
	^ self new register! !

MagmaWarning subclass: #MagmaToolsWarning
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma-Tools-Exceptions'!

!MaObjectBuffer methodsFor: '*magma-tools' stamp: 'cmm 3/19/2007 00:26'!
hasValidReferencesFor: aMaObjectFiler
	^ true! !

!MaObjectBuffer methodsFor: '*magma-tools' stamp: 'cmm 8/4/2011 20:34'!
isValidBufferType
	^ self type between: 1 and: MaObjectBuffer classes size! !

!MaObjectBuffer methodsFor: '*magma-tools' stamp: 'cmm 3/19/2007 12:21'!
isValidOid: anInteger for: aMaObjectFiler 
	^ anInteger
		between: MaOidCalculator firstUserObjectOid
		and: aMaObjectFiler getNextOid - 1! !

!MaObjectBuffer methodsFor: '*magma-tools' stamp: 'cmm 3/19/2007 12:21'!
isValidOidFor: aMaObjectFiler 
	^ self 
		isValidOid: self oid
		for: aMaObjectFiler! !

!MaFixedObjectBuffer methodsFor: '*magma-tools' stamp: 'cmm 3/19/2007 12:25'!
hasValidReferencesFor: aMaObjectFiler 
	self instVarsDoWithIndex: 
		[ : eachOid : x | 
		(self 
			isValidOid: eachOid
			for: aMaObjectFiler) ifFalse: [ ^ false ] ].
	^ true! !
MagmaToolsPackageInfo initialize!
